from oauthlib.oauth2 import OAuth2Token


class OAuth2Model:
    _token: OAuth2Token | None

    def __init__(self) -> None:
        self._token = None

    @property
    def token(self) -> OAuth2Token | None:
        return self._token

    @token.setter
    def token(self, token: OAuth2Token | None) -> None:
        self._token = token
