#!/usr/bin/python3

from clochette.Clochette import Clochette
from clochette.infrastructure.LoggingService import LoggingService
from clochette.infrastructure.inject.Container import Container
from clochette.presentation.QMainApplication import QMainApplication
from clochette.presentation.window.QWebBrowserWindow import QWebBrowserWindow


def main():
    container = Container()
    _ = container.instantiate(QMainApplication)

    container.instantiate(LoggingService).setup()
    container.instantiate(QWebBrowserWindow)
    container.instantiate(Clochette).start()


if __name__ == "__main__":
    main()
