from dataclasses import dataclass

from oauthlib.oauth2 import OAuth2Token

from clochette.provider.microsoft.dto.MicrosoftCalendar import MicrosoftCalendar


@dataclass(frozen=True)
class MicrosoftCalendarList:
    calendars: list[MicrosoftCalendar]
    token: OAuth2Token
