from dataclasses import dataclass

from oauthlib.oauth2 import OAuth2Token

from clochette.provider.google.dto.GoogleCalendar import GoogleCalendar


@dataclass(frozen=True)
class GoogleCalendarList:
    calendars: list[GoogleCalendar]
    token: OAuth2Token
