from dataclasses import dataclass

from clochette.presentation.reminder_window.QReminderWindowComponent import (
    QReminderWindowComponent,
)


@dataclass
class InitializeReminderWindowUseCase:
    """Use case to initialize and start the reminder window component."""

    _reminder_window_component: QReminderWindowComponent

    def initialize(self) -> None:
        """Initialize the reminder window by subscribing to occurrence updates."""
        self._reminder_window_component.start()
