from clochette.provider.shared.ui.QOAuth2View import QOAuth2View


class QOAuth2ViewFactory:
    """Factory for creating QOAuth2View instances with specific calendar types."""

    @staticmethod
    def create() -> QOAuth2View:
        """Creates a new QOAuth2View instance."""
        return QOAuth2View()
