from dataclasses import dataclass

from oauthlib.oauth2 import OAuth2Token

from clochette.domain.entity.ICredentialCalendar import ICredentialCalendar


@dataclass(frozen=True)
class GoogleCredentials(ICredentialCalendar):
    token: OAuth2Token
