from dataclasses import dataclass

from clochette.domain.entity.ICredentialCalendar import ICredentialCalendar


@dataclass(frozen=True)
class BasicCredentials(ICredentialCalendar):
    username: str
    password: str
